@@ -12,6 +12,6 @@ from palabra_ai import (
 if __name__ == "__main__":
     palabra = PalabraAI()
     reader = FileReader("./speech/es.mp3")
-    writer = FileWriter("./es2en_out.wav")
+    writer = FileWriter("./sep23_es2en_out.wav")
     cfg = Config(SourceLang(ES, reader), [TargetLang(EN, writer)], debug=True)
     palabra.run(cfg)
