@@ -3,17 +3,27 @@ import ctypes
 import io
 import wave
 from dataclasses import dataclass, field
+from pathlib import Path
 from typing import Optional
 
 import numpy as np
 from livekit.rtc import AudioFrame as RtcAudioFrame
 
-from palabra_ai.constant import BYTES_PER_SAMPLE
+from palabra_ai.constant import AUDIO_BUFFER_PADDING_SEC, BYTES_PER_SAMPLE
 from palabra_ai.util.logger import error
 from palabra_ai.util.orjson import from_json, to_json
 from palabra_ai.util.timing import get_perf_ts
 
 
+def save_wav(np_audio: np.typing.NDArray, output_path: Path, sr: int, ch: int):
+    """Save audio chunks to WAV file"""
+    with wave.open(str(output_path), "wb") as wav:
+        wav.setnchannels(ch)
+        wav.setframerate(sr)
+        wav.setsampwidth(BYTES_PER_SAMPLE)
+        wav.writeframes(np_audio.tobytes())
+
+
 class AudioFrame:
     """Lightweight AudioFrame replacement with __slots__ for performance"""
 
@@ -135,7 +145,7 @@ class AudioFrame:
         )
 
     def __repr__(self):
-        return f"🗣️<AF(s={self.samples_per_channel}, sr={self.sample_rate}, ch={self.num_channels})>"
+        return f"🗣️<AF(dur={self.duration:.3f}s, s={self.samples_per_channel}, sr={self.sample_rate}, ch={self.num_channels})>"
 
     def __bool__(self):
         """Return False if data is empty, True otherwise"""
@@ -164,7 +174,6 @@ class AudioFrame:
         raw_msg: bytes | str,
         sample_rate: int,
         num_channels: int,
-        samples_per_channel: int,
         perf_ts: float | None = None,
     ) -> Optional["AudioFrame"]:
         """Create AudioFrame from WebSocket message
@@ -208,6 +217,8 @@ class AudioFrame:
             # Decode base64 to bytes
             audio_bytes = base64.b64decode(base64_data)
 
+            samples_per_channel = len(audio_bytes) // (BYTES_PER_SAMPLE * num_channels)
+
             return cls(
                 data=audio_bytes,
                 sample_rate=sample_rate,
@@ -286,10 +297,13 @@ class AudioBuffer:
     audio_array: np.ndarray = field(default=None, init=False)
     last_audio_end_position: int = field(default=0, init=False)
     global_start_time: float | None = field(default=None, init=False)
+    external_buffer: io.BytesIO | None = field(default=None, init=False)
 
     def __post_init__(self):
-        # Create numpy array like file2file - duration + 60s buffer
-        total_samples = int((self.original_duration + 60) * self.sample_rate)
+        # Create numpy array like file2file - duration + buffer padding
+        total_samples = int(
+            (self.original_duration + AUDIO_BUFFER_PADDING_SEC) * self.sample_rate
+        )
         self.audio_array = np.zeros(total_samples, dtype=np.int16)
 
     def set_start_time(self, start_time: float):
@@ -297,6 +311,10 @@ class AudioBuffer:
         if self.global_start_time is None:
             self.global_start_time = start_time
 
+    def replace_buffer(self, external_buffer: io.BytesIO):
+        """Replace internal buffer with external BytesIO buffer for direct writing"""
+        self.external_buffer = external_buffer
+
     def to_wav_bytes(self) -> bytes:
         """Convert buffer to WAV format"""
         if self.audio_array is None or len(self.audio_array) == 0:
@@ -315,13 +333,25 @@ class AudioBuffer:
             : last_sample + int(self.sample_rate * 2)
         ]  # +2s buffer
 
-        with io.BytesIO() as wav_file:
-            with wave.open(wav_file, "wb") as wav:
+        # If external buffer is set, write directly to it
+        if self.external_buffer is not None:
+            self.external_buffer.seek(0)
+            with wave.open(self.external_buffer, "wb") as wav:
                 wav.setnchannels(self.num_channels)
                 wav.setframerate(self.sample_rate)
                 wav.setsampwidth(BYTES_PER_SAMPLE)
                 wav.writeframes(trimmed_array.tobytes())
-            return wav_file.getvalue()
+            self.external_buffer.seek(0)
+            return self.external_buffer.getvalue()
+        else:
+            # Default behavior: return bytes
+            with io.BytesIO() as wav_file:
+                with wave.open(wav_file, "wb") as wav:
+                    wav.setnchannels(self.num_channels)
+                    wav.setframerate(self.sample_rate)
+                    wav.setsampwidth(BYTES_PER_SAMPLE)
+                    wav.writeframes(trimmed_array.tobytes())
+                return wav_file.getvalue()
 
     async def write(self, frame: AudioFrame):
         """Write frame to buffer using file2file approach"""
