@@ -14,7 +14,7 @@ from palabra_ai.internal.rest import PalabraRESTClient, SessionCredentials
 from palabra_ai.model import RunResult
 from palabra_ai.task.base import TaskEvent
 from palabra_ai.task.manager import Manager
-from palabra_ai.util.logger import debug, error, success
+from palabra_ai.util.logger import debug, error, exception, success
 
 
 @dataclass
@@ -79,7 +79,7 @@ class PalabraAI:
             debug("Received keyboard interrupt (Ctrl+C)")
             return None
         except BaseException as e:
-            error(f"An error occurred during execution: {e}")
+            exception("An error occurred during execution")
             if no_raise:
                 return RunResult(ok=False, exc=e, eos=False)
             raise e
@@ -91,7 +91,7 @@ class PalabraAI:
         cfg: Config,
         stopper: TaskEvent | None = None,
         no_raise=False,
-    ) -> RunResult | None:
+    ) -> RunResult:
         """
         Run the translation asynchronously (returns awaitable coroutine).
 
@@ -101,7 +101,9 @@ class PalabraAI:
             no_raise: If True, return RunResult with exception instead of raising
 
         Returns:
-            RunResult with execution status and logs, or None if cancelled
+            RunResult with execution status and logs
+            - If no_raise=False: returns RunResult(ok=True) or raises exception
+            - If no_raise=True: always returns RunResult (ok=True or ok=False with exc)
         """
 
         async def _run_with_result(manager: Manager) -> RunResult:
@@ -113,10 +115,16 @@ class PalabraAI:
                 await manager.task
                 ok = True
             except asyncio.CancelledError as e:
-                debug("Manager task was cancelled")
-                exc = e
+                # Check if this is graceful shutdown or external cancellation
+                if manager._graceful_completion:
+                    debug("Manager task cancelled during graceful shutdown")
+                    ok = True  # Graceful shutdown is successful completion
+                    # exc remains None - not an error
+                else:
+                    exception("Manager task was cancelled")
+                    exc = e
             except BaseException as e:
-                error(f"Error in manager task: {e}")
+                exception("Error in manager task")
                 exc = e
 
             # CRITICAL: Always try to get log_data from logger
@@ -142,22 +150,31 @@ class PalabraAI:
                             )
                         except Exception as e:
                             debug(f"Failed to get log_data from logger.exit(): {e}")
-            except Exception as e:
-                error(f"Failed to retrieve log_data: {e}")
+            except Exception:
+                exception("Failed to retrieve log_data")
 
             # Check if EOS was received (only relevant for WS)
             eos_received = manager.io.eos_received if manager.io else False
 
             # Return result with whatever we managed to get
-            if no_raise or ok:
+            if no_raise:
                 return RunResult(
                     ok=ok,
                     exc=exc if not ok else None,
                     log_data=log_data,
+                    io_data=manager.io.io_data if manager.io else None,
                     eos=eos_received,
                 )
-            elif exc:
-                # Save log_data before raising exception
+            elif ok:
+                return RunResult(
+                    ok=True,
+                    exc=None,
+                    log_data=log_data,
+                    io_data=manager.io.io_data if manager.io else None,
+                    eos=eos_received,
+                )
+            else:
+                # no_raise=False and there was an error - raise it
                 raise exc
 
         try:
@@ -168,11 +185,26 @@ class PalabraAI:
                 result = await coro
                 if DEEP_DEBUG:
                     debug(diagnose_hanging_tasks())
+
+                # Ensure result is not None
+                if result is None:
+                    # This should not happen, but just in case
+                    if no_raise:
+                        return RunResult(
+                            ok=False,
+                            exc=RuntimeError("Unexpected None result"),
+                            eos=False,
+                        )
+                    raise RuntimeError("Unexpected None result from _run_with_result")
+
                 return result
+
         except BaseException as e:
-            error(f"Error in PalabraAI.arun(): {e}")
+            exception("Error in PalabraAI.arun()")
+            # When no_raise=True, ALWAYS return RunResult with ok=False
             if no_raise:
                 return RunResult(ok=False, exc=e, eos=False)
+            # When no_raise=False, ALWAYS raise exception
             raise
         finally:
             if DEEP_DEBUG:
@@ -201,22 +233,30 @@ class PalabraAI:
             credentials = await rest_client.create_session()
             session_created_internally = True
 
-        manager = None
         try:
             async with asyncio.TaskGroup() as tg:
                 manager = Manager(cfg, credentials, stopper=stopper)(tg)
                 yield manager
             success("🎉🎉🎉 Translation completed 🎉🎉🎉")
 
-        except* asyncio.CancelledError:
-            debug("TaskGroup received CancelledError")
+        except* asyncio.CancelledError as eg:
+            # Check if this was graceful completion (internal shutdown)
+            if manager._graceful_completion:
+                debug("Graceful completion detected - not propagating CancelledError")
+                # Don't re-raise - this is expected graceful shutdown
+            else:
+                # External cancellation (Ctrl+C, timeout, error)
+                debug("External cancellation - propagating CancelledError")
+                raise eg.exceptions[0] from eg
         except* Exception as eg:
             excs = unwrap_exceptions(eg)
             excs_wo_cancel = [
                 e for e in excs if not isinstance(e, asyncio.CancelledError)
             ]
-            for e in excs:
-                error(f"Unhandled exception: {e}")
+            if excs_wo_cancel:
+                exception(
+                    f"Unhandled exception in TaskGroup: {len(excs_wo_cancel)} error(s)"
+                )
             if not excs_wo_cancel:
                 raise excs[0] from eg
             raise excs_wo_cancel[0] from eg
@@ -230,7 +270,7 @@ class PalabraAI:
                     success(f"Successfully deleted session {credentials.id}")
                 except TimeoutError:
                     error(f"Timeout deleting session {credentials.id}")
-                except Exception as e:
-                    error(f"Failed to delete session {credentials.id}: {e}")
+                except Exception:
+                    exception(f"Failed to delete session {credentials.id}")
 
             debug(diagnose_hanging_tasks())
