@@ -35,8 +35,10 @@ QUEUE_WAIT_TIMEOUT = 0.5
 WS_TIMEOUT = 5.0
 DEBUG_TASK_CHECK_INTERVAL = 30.0
 
+RT_DEFAULT_DURATION = 60.0
 
 # Buffer and Queue Constants
+AUDIO_BUFFER_PADDING_SEC = 60  # Extra buffer time in seconds
 THREADPOOL_MAX_WORKERS = 32
 DEVICE_ID_HASH_LENGTH = 8
 MONITOR_MESSAGE_PREVIEW_LENGTH = 100
@@ -49,6 +51,8 @@ COMPLETION_WAIT_TIMEOUT = 2.0
 STATS_LOG_INTERVAL = 5.0
 
 # Preprocessing Constants
+FORCE_SPLIT_MIN_CHARACTERS_DEFAULT = 100
+FORCE_SPLIT_MIN_SECONDS_DEFAULT = 6.0
 MIN_SENTENCE_CHARACTERS_DEFAULT = 80
 MIN_SENTENCE_SECONDS_DEFAULT = 4
 MIN_SPLIT_INTERVAL_DEFAULT = 0.6
@@ -69,10 +73,13 @@ ENERGY_VARIANCE_FACTOR_DEFAULT = 1.5
 SPEECH_TEMPO_ADJUSTMENT_FACTOR_DEFAULT = 0.75
 
 # Queue Config Constants
-DESIRED_QUEUE_LEVEL_MS_DEFAULT = 10000
-MAX_QUEUE_LEVEL_MS_DEFAULT = 24000
-QUEUE_MIN_TEMPO = 1.0
-QUEUE_MAX_TEMPO = 1.2
+DESIRED_QUEUE_LEVEL_MS_DEFAULT = 5000
+MAX_QUEUE_LEVEL_MS_DEFAULT = 20000
+QUEUE_MIN_TEMPO = 1.15
+QUEUE_MAX_TEMPO = 1.45
+AUTO_TEMPO_MAX_DELAY_MS_DEFAULT = 250
+TEMPO_DECAY_DEFAULT = 0.35
+TEMPO_SMOOTHING_DEFAULT = 0.005
 
 # TranscriptionMessage Constants
 MIN_ALIGNMENT_SCORE_DEFAULT = 0.2
@@ -94,3 +101,5 @@ WEBRTC_MODE_INPUT_SAMPLE_RATE = 48000
 WEBRTC_MODE_OUTPUT_SAMPLE_RATE = 48000
 WEBRTC_MODE_CHANNELS = 1
 WEBRTC_MODE_CHUNK_DURATION_MS = 320
+
+EOF_SILENCE_DURATION_S = 10.0
