@@ -25,6 +25,7 @@ class Reader(Task):
     q: asyncio.Queue[AudioFrame] = field(default_factory=asyncio.Queue)
     # chunk_size: int = CHUNK_SIZE
     eof: TaskEvent = field(default_factory=TaskEvent, init=False)
+    duration: float | None = field(default=None, init=False)
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -98,20 +99,29 @@ class Writer(Task):
         return await super()._exit()
 
 
+class UnlimitedExitMixin:
+    """Mixin to allow unlimited time for exit operations (file/buffer saving)."""
+
+    async def _exit(self):
+        """Override to allow unlimited time for saving operations"""
+        self.q.put_nowait(None)
+        debug(f"{self.name}._exit() allowing unlimited time for save operation...")
+        return await self.exit()
+
+
 @dataclass
-class BufferedWriter(Writer):
+class BufferedWriter(UnlimitedExitMixin, Writer):
     """Writer that buffers audio frames before writing."""
 
     _: KW_ONLY
     ab: AudioBuffer | None = field(default=None, init=False)
 
     async def boot(self):
-        # Create buffer with estimated duration
-        estimated_duration = getattr(self.cfg, "estimated_duration", 60.0)
+        # Create buffer with estimated duration from config
         self.ab = AudioBuffer(
             sample_rate=self.cfg.mode.output_sample_rate,
             num_channels=self.cfg.mode.num_channels,
-            original_duration=estimated_duration,
+            original_duration=self.cfg.estimated_duration,
             drop_empty_frames=getattr(self.cfg, "drop_empty_frames", False),
         )
 
