@@ -44,7 +44,7 @@ class FileReader(Reader):
     _iterator: Iterator[av.AudioFrame] | None = None
     _buffer: deque = None
     _position: int = 0
-    _total_duration: float = 0.0
+
     _target_rate: int = 0
     _preprocessed: bool = False
 
@@ -75,13 +75,14 @@ class FileReader(Reader):
                 target_rate=self.cfg.mode.input_sample_rate,
                 normalize=normalize,
                 progress_callback=progress_callback,
+                eof_silence_duration_s=self.cfg.mode.eof_silence_duration_s,
             )
             # Simple mode uses config as-is
             debug(
                 f"Simple mode: using config sample rate {self.cfg.mode.input_sample_rate}Hz"
             )
 
-            self._total_duration = metadata["duration"]
+            self.duration = metadata["duration"]
             self._target_rate = metadata["final_rate"]
 
             # Split into chunks and store in buffer
@@ -122,7 +123,7 @@ class FileReader(Reader):
                 f"Simple streaming: using config sample rate {self.cfg.mode.input_sample_rate}Hz"
             )
 
-            self._total_duration = metadata["duration"]
+            self.duration = metadata["duration"]
 
             # Create iterator but don't start reading yet
             self._iterator = self._container.decode(audio=0)
@@ -136,9 +137,7 @@ class FileReader(Reader):
     async def exit(self):
         seconds_processed = self._position / (self._target_rate * BYTES_PER_SAMPLE)
         progress_pct = (
-            (seconds_processed / self._total_duration) * 100
-            if self._total_duration > 0
-            else 0
+            (seconds_processed / self.duration) * 100 if self.duration > 0 else 0
         )
         debug(f"{self.name} processed {seconds_processed:.1f}s ({progress_pct:.1f}%)")
 
@@ -186,7 +185,7 @@ class FileReader(Reader):
             return  # Already enough data
 
         # Read a few frames to fill buffer
-        chunk_bytes = self.cfg.mode.chunk_bytes
+        chunk_bytes = self.cfg.mode.input_chunk_bytes
         frames_to_read = max(1, (needed_size - current_size) // chunk_bytes + 1)
 
         for _ in range(
