@@ -102,8 +102,9 @@ class WebrtcIo(Io):
                         Direction.OUT,
                         idx=next(self._idx),
                         num=next(self._out_audio_num),
-                        chunk_duration_ms=self.cfg.mode.chunk_duration_ms,
+                        dur_s=audio_frame.duration,
                     )
+                    audio_frame._dbg.calc_dawn_ts(self.global_start_perf_ts)
                     self.bench_audio_foq.publish(audio_frame)
                 await aio.sleep(0)
                 if self.stopper or self.eof:
@@ -123,6 +124,7 @@ class WebrtcIo(Io):
             idx=next(self._idx),
             num=next(self._out_msg_num),
         )
+        _dbg.calc_dawn_ts(self.global_start_perf_ts)
         debug(f"Received packet: {data}"[:100])
         msg = Message.decode(data.data)
         msg._dbg = _dbg
@@ -140,7 +142,7 @@ class WebrtcIo(Io):
     async def send_message(self, msg_data: bytes) -> None:
         await self.room.local_participant.publish_data(msg_data, reliable=True)
 
-    async def send_frame(self, frame: AudioFrame) -> None:
+    async def send_frame(self, frame: AudioFrame, raw: bytes | None = None) -> None:
         self.init_global_start_ts()
         return await self.in_audio_source.capture_frame(frame.to_rtc())
 
@@ -167,7 +169,7 @@ class WebrtcIo(Io):
         # noinspection PyTypeChecker
         self.in_track_options.source = self.track_source
         self.in_audio_source = rtc.AudioSource(
-            self.cfg.mode.sample_rate, self.cfg.mode.num_channels
+            self.cfg.mode.input_sample_rate, self.cfg.mode.num_channels
         )
         self.in_track = rtc.LocalAudioTrack.create_audio_track(
             self.in_track_name, self.in_audio_source
