@@ -5,7 +5,7 @@ from websockets.asyncio.client import connect as ws_connect
 
 from palabra_ai.audio import AudioFrame
 from palabra_ai.enum import Channel, Direction, Kind
-from palabra_ai.message import Dbg
+from palabra_ai.message import Dbg, EosMessage, IoEvent, Message
 from palabra_ai.task.io.base import Io
 from palabra_ai.util.logger import debug, trace
 from palabra_ai.util.timing import get_perf_ts, get_utc_ts
@@ -28,20 +28,26 @@ class WsIo(Io):
     async def send_message(self, msg_data: bytes) -> None:
         await self.ws.send(msg_data)
 
-    async def send_frame(self, frame: AudioFrame) -> None:
-        raw = frame.to_ws()
+    async def send_frame(self, frame: AudioFrame, raw: bytes | None = None) -> None:
+        if not raw:
+            raw = frame.to_ws()
         debug(f"<- {frame} / {frame.dbg_delta=}")
-        self.init_global_start_ts()
+        if self.global_start_perf_ts is None:
+            self.init_global_start_ts()
+            if hasattr(frame, "_dbg") and frame._dbg:
+                frame._dbg.dawn_ts = 0.0
         await self.ws.send(raw)
 
-    def new_frame(self) -> AudioFrame:
-        return AudioFrame.create(*self.cfg.mode.for_audio_frame)
+    def new_input_frame(self) -> AudioFrame:
+        return AudioFrame.create(*self.cfg.mode.for_input_audio_frame)
 
     async def ws_receiver(self):
-        from palabra_ai.message import EosMessage, Message
-
         try:
             async for raw_msg in self.ws:
+                # Check for cancellation/stopper eagerly
+                if self.stopper:
+                    debug("ws_receiver: stopper detected, exiting")
+                    break
                 perf_ts = get_perf_ts()
                 utc_ts = get_utc_ts()
                 if self.stopper or raw_msg is None:
@@ -52,7 +58,6 @@ class WsIo(Io):
                     raw_msg,
                     sample_rate=self.cfg.mode.output_sample_rate,
                     num_channels=self.cfg.mode.num_channels,
-                    samples_per_channel=self.cfg.mode.samples_per_channel,
                     perf_ts=perf_ts,
                 )
                 if audio_frame:
@@ -64,12 +69,14 @@ class WsIo(Io):
                             Direction.OUT,
                             idx=next(self._idx),
                             num=next(self._out_audio_num),
-                            chunk_duration_ms=self.cfg.mode.chunk_duration_ms,
+                            dur_s=audio_frame.duration,
                             perf_ts=perf_ts,
                             utc_ts=utc_ts,
                         )
+                        _dbg.calc_dawn_ts(self.global_start_perf_ts)
                         audio_frame._dbg = _dbg
                         self.bench_audio_foq.publish(audio_frame)
+                        self.io_events.append(IoEvent(_dbg, raw_msg))
                     self.writer.q.put_nowait(audio_frame)
                 else:
                     _dbg = Dbg(
@@ -79,9 +86,11 @@ class WsIo(Io):
                         idx=next(self._idx),
                         num=next(self._out_audio_num),
                     )
+                    _dbg.calc_dawn_ts(self.global_start_perf_ts)
                     msg = Message.decode(raw_msg)
                     msg._dbg = _dbg
                     self.out_msg_foq.publish(msg)
+                    self.io_events.append(IoEvent(_dbg, raw_msg))
                     debug(f"-> {msg!r}")
                     if isinstance(msg, EosMessage):
                         self.eos_received = True
@@ -109,6 +118,12 @@ class WsIo(Io):
 
     async def exit(self):
         """Clean up WebSocket connection"""
-        if self._ws_cm and self.ws:
-            await self._ws_cm.__aexit__(None, None, None)
+        # Websocket should already be closed in do(), just cleanup context manager
+        if self._ws_cm:
+            try:
+                await self._ws_cm.__aexit__(None, None, None)
+                debug(f"{self.name}: WebSocket context manager cleaned up")
+            except Exception as e:
+                debug(f"{self.name}: Error exiting websocket context: {e}")
+
         self.ws = None
