@@ -5,6 +5,7 @@ from palabra_ai.client import PalabraAI
 from palabra_ai.config import Config, SourceLang, TargetLang
 from palabra_ai.exc import ConfigurationError
 from palabra_ai.task.base import TaskEvent
+from palabra_ai.model import RunResult
 
 def test_palabra_ai_creation():
     """Test PalabraAI client creation with credentials"""
@@ -61,6 +62,7 @@ def test_run_without_loop():
     # Mock the async context manager directly to avoid coroutine creation
     with patch.object(client, 'process') as mock_process:
         mock_manager = MagicMock()
+        mock_manager.io_data = {}
         mock_manager.logger = None
         mock_async_context = AsyncMock()
         mock_async_context.__aenter__.return_value = mock_manager
@@ -249,6 +251,7 @@ async def test_process_with_credentials_creation():
 
         with patch('palabra_ai.client.Manager') as mock_manager_class:
             mock_manager = MagicMock()
+            mock_manager.io_data = {}
             mock_manager_class.return_value = MagicMock(return_value=mock_manager)
 
             with patch('asyncio.TaskGroup') as mock_tg_class:
@@ -284,6 +287,8 @@ async def test_process_with_cancelled_error():
 
         with patch('palabra_ai.client.Manager') as mock_manager_class:
             mock_manager = MagicMock()
+            mock_manager.io_data = {}
+            mock_manager._graceful_completion = False  # NOT graceful - external cancel
             mock_manager_class.return_value = MagicMock(return_value=mock_manager)
 
             with patch('asyncio.TaskGroup') as mock_tg_class:
@@ -293,9 +298,10 @@ async def test_process_with_cancelled_error():
                 mock_tg.__aexit__.side_effect = asyncio.CancelledError()
                 mock_tg_class.return_value = mock_tg
 
-                # Should not raise CancelledError
-                async with client.process(config) as manager:
-                    pass
+                # Should properly re-raise CancelledError from TaskGroup (not graceful)
+                with pytest.raises(asyncio.CancelledError):
+                    async with client.process(config) as manager:
+                        pass
 
 @pytest.mark.asyncio
 async def test_process_with_exception_group():
@@ -315,6 +321,7 @@ async def test_process_with_exception_group():
 
         with patch('palabra_ai.client.Manager') as mock_manager_class:
             mock_manager = MagicMock()
+            mock_manager.io_data = {}
             mock_manager_class.return_value = MagicMock(return_value=mock_manager)
 
             with patch('asyncio.TaskGroup') as mock_tg_class:
@@ -365,6 +372,7 @@ async def test_process_finally_block():
 
             with patch('palabra_ai.client.Manager') as mock_manager_class:
                 mock_manager = MagicMock()
+                mock_manager.io_data = {}
                 mock_manager_class.return_value = MagicMock(return_value=mock_manager)
 
                 with patch('asyncio.TaskGroup') as mock_tg_class:
@@ -419,6 +427,7 @@ def test_run_without_signal_handlers():
 
             with patch.object(client, 'process') as mock_process:
                 mock_manager = MagicMock()
+                mock_manager.io_data = {}
                 mock_manager.task = MagicMock()
                 mock_manager.logger = None
                 mock_async_context = AsyncMock()
@@ -433,7 +442,7 @@ def test_run_without_signal_handlers():
                 mock_loop.close.assert_called_once()
 
 def test_run_async_with_manager_task_cancelled():
-    """Test _run_with_result when manager task is cancelled"""
+    """Test _run_with_result when manager task is cancelled (external cancellation)"""
     config = Config()
     config.source = SourceLang(lang="es")
 
@@ -442,6 +451,10 @@ def test_run_async_with_manager_task_cancelled():
     async def test_coro():
         with patch.object(client, 'process') as mock_process:
             mock_manager = MagicMock()
+            mock_manager._graceful_completion = False  # External cancellation, not graceful
+            mock_io = MagicMock()
+            mock_io.io_data = {"start_perf_ts": 0.0, "start_utc_ts": 0.0, "in_sr": 16000, "out_sr": 16000, "mode": "test", "channels": 1, "events": [], "count_events": 0}
+            mock_manager.io = mock_io
             # Create a future that raises CancelledError when awaited
             mock_task = asyncio.Future()
             mock_task.set_exception(asyncio.CancelledError())
@@ -470,6 +483,10 @@ def test_run_async_with_manager_error():
     async def test_coro():
         with patch.object(client, 'process') as mock_process:
             mock_manager = MagicMock()
+            mock_io = MagicMock()
+            mock_io.io_data = {"start_perf_ts": 0.0, "start_utc_ts": 0.0, "in_sr": 16000, "out_sr": 16000, "mode": "test", "channels": 1, "events": [], "count_events": 0}
+            mock_io.eos_received = False
+            mock_manager.io = mock_io
             # Create a future that raises ValueError when awaited
             mock_task = asyncio.Future()
             mock_task.set_exception(ValueError("Manager error"))
@@ -482,12 +499,12 @@ def test_run_async_with_manager_error():
             mock_process.return_value.__aenter__.return_value = mock_manager
             mock_process.return_value.__aexit__.return_value = None
 
-            with patch('palabra_ai.client.error') as mock_error:
+            with patch('palabra_ai.client.exception') as mock_exception:
                 result = await client.arun(config, no_raise=True)
                 assert result.ok is False
                 assert isinstance(result.exc, ValueError)
                 assert result.log_data is None
-                mock_error.assert_any_call("Error in manager task: Manager error")
+                mock_exception.assert_any_call("Error in manager task")
 
     asyncio.run(test_coro())
 
@@ -501,6 +518,9 @@ def test_run_async_with_logger_timeout():
     async def test_coro():
         with patch.object(client, 'process') as mock_process:
             mock_manager = MagicMock()
+            mock_io = MagicMock()
+            mock_io.io_data = {"start_perf_ts": 0.0, "start_utc_ts": 0.0, "in_sr": 16000, "out_sr": 16000, "mode": "test", "channels": 1, "events": [], "count_events": 0}
+            mock_manager.io = mock_io
             async def normal_task():
                 return None
             mock_manager.task = asyncio.create_task(normal_task())
@@ -531,6 +551,9 @@ def test_run_async_with_logger_exception():
     async def test_coro():
         with patch.object(client, 'process') as mock_process:
             mock_manager = MagicMock()
+            mock_io = MagicMock()
+            mock_io.io_data = {"start_perf_ts": 0.0, "start_utc_ts": 0.0, "in_sr": 16000, "out_sr": 16000, "mode": "test", "channels": 1, "events": [], "count_events": 0}
+            mock_manager.io = mock_io
             # Create a future that completes normally
             mock_task = asyncio.Future()
             mock_task.set_result(None)
@@ -563,11 +586,11 @@ def test_run_async_with_process_error():
         with patch.object(client, 'process') as mock_process:
             mock_process.side_effect = RuntimeError("Process error")
 
-            with patch('palabra_ai.client.error') as mock_error:
+            with patch('palabra_ai.client.exception') as mock_exception:
                 result = await client.arun(config, no_raise=True)
                 assert result.ok is False
                 assert isinstance(result.exc, RuntimeError)
-                mock_error.assert_called_with("Error in PalabraAI.arun(): Process error")
+                mock_exception.assert_called_with("Error in PalabraAI.arun()")
     asyncio.run(test_coro())
 
 # Tests for new arun() method
@@ -655,6 +678,178 @@ def test_run_signal_handlers_true():
                 # Should have used signal context manager
                 mock_context.run_until_complete.assert_called_once()
 
+class TestErrorHandlingNoRaise:
+    """Test error handling for both run() and arun() methods with no_raise parameter."""
+
+    @pytest.fixture
+    def mock_config(self):
+        """Create test configuration."""
+        from palabra_ai.config import SourceLang, TargetLang
+        from palabra_ai.lang import EN, ES
+        from palabra_ai.task.adapter.dummy import DummyReader, DummyWriter
+        from palabra_ai.config import WsMode
+
+        cfg = Config(
+            SourceLang(EN, DummyReader()),
+            [TargetLang(ES, DummyWriter())],
+            mode=WsMode(),
+            silent=True,
+            estimated_duration=5.0,
+        )
+        return cfg
+
+    @pytest.mark.asyncio
+    async def test_arun_no_raise_false_with_error_should_raise(self, mock_config):
+        """Test arun with no_raise=False should RAISE exception when there's an error."""
+        client = PalabraAI(client_id="test", client_secret="test")
+
+        # Mock process() to fail directly
+        with patch.object(client, 'process') as mock_process:
+            mock_process.side_effect = asyncio.TimeoutError("Process failed")
+
+            # This should RAISE exception, not return RunResult
+            try:
+                result = await client.arun(mock_config, no_raise=False)
+                # If we reach this line, the bug still exists
+                assert False, f"Expected TimeoutError to be raised, but got RunResult: {result}"
+            except asyncio.TimeoutError as e:
+                # This is what should happen
+                assert "Process failed" in str(e)
+
+    @pytest.mark.asyncio
+    async def test_arun_no_raise_true_with_error_should_return_result(self, mock_config):
+        """Test arun with no_raise=True should return RunResult(ok=False) when there's an error."""
+        client = PalabraAI(client_id="test", client_secret="test")
+
+        # Mock process() to fail directly
+        with patch.object(client, 'process') as mock_process:
+            mock_process.side_effect = asyncio.TimeoutError("Process failed")
+
+            result = await client.arun(mock_config, no_raise=True)
+
+            assert result is not None
+            assert result.ok is False
+            assert result.exc is not None
+            assert isinstance(result.exc, asyncio.TimeoutError)
+            assert "Process failed" in str(result.exc)
+
+    @pytest.mark.asyncio
+    async def test_arun_no_raise_false_with_cancelled_should_raise(self, mock_config):
+        """Test arun with no_raise=False should RAISE CancelledError when task is cancelled."""
+        client = PalabraAI(client_id="test", client_secret="test")
+
+        # Mock process() context manager to simulate cancellation
+        with patch.object(client, 'process') as mock_process:
+            mock_process.side_effect = asyncio.CancelledError("Task was cancelled")
+
+            # This should RAISE CancelledError, not return None or RunResult
+            with pytest.raises(asyncio.CancelledError) as exc_info:
+                await client.arun(mock_config, no_raise=False)
+
+            assert "Task was cancelled" in str(exc_info.value)
+
+    @pytest.mark.asyncio
+    async def test_arun_no_raise_true_with_cancelled_should_return_result(self, mock_config):
+        """Test arun with no_raise=True should return RunResult(ok=False) when cancelled."""
+        client = PalabraAI(client_id="test", client_secret="test")
+
+        # Mock process() to raise CancelledError
+        with patch.object(client, 'process') as mock_process:
+            mock_process.side_effect = asyncio.CancelledError("Task was cancelled")
+
+            result = await client.arun(mock_config, no_raise=True)
+
+            # CRITICAL: Must return RunResult, NOT None!
+            assert result is not None, "arun() returned None instead of RunResult"
+            assert result.ok is False
+            assert result.exc is not None
+            assert isinstance(result.exc, asyncio.CancelledError)
+            assert "Task was cancelled" in str(result.exc)
+
+    @pytest.mark.asyncio
+    async def test_arun_never_returns_none_with_no_raise_true(self, mock_config):
+        """Test that arun NEVER returns None when no_raise=True, even with weird errors."""
+        client = PalabraAI(client_id="test", client_secret="test")
+
+        # Test various exception types
+        exceptions_to_test = [
+            asyncio.CancelledError("Cancelled"),
+            asyncio.TimeoutError("Timeout"),
+            ConnectionError("Connection failed"),
+            RuntimeError("Random error"),
+            Exception("Generic exception"),
+        ]
+
+        for exc in exceptions_to_test:
+            with patch.object(client, 'process') as mock_process:
+                mock_process.side_effect = exc
+
+                result = await client.arun(mock_config, no_raise=True)
+
+                assert result is not None, f"arun() returned None for {type(exc).__name__}"
+                assert isinstance(result, RunResult), f"Expected RunResult, got {type(result)}"
+                assert result.ok is False
+                assert result.exc is exc
+
+    @pytest.mark.asyncio
+    async def test_arun_process_swallows_cancelled_error(self, mock_config):
+        """Test what happens when process() context manager fails to yield proper manager."""
+        client = PalabraAI(client_id="test", client_secret="test")
+
+        # Create a broken context manager that yields None instead of proper manager
+        async def broken_process_context(*args, **kwargs):
+            class BrokenContext:
+                async def __aenter__(self):
+                    # Simulate process() returning None/invalid manager due to suppressed CancelledError
+                    return None  # This is the bug!
+                async def __aexit__(self, exc_type, exc_val, exc_tb):
+                    return False
+            return BrokenContext()
+
+        with patch.object(client, 'process', side_effect=broken_process_context):
+            # This should expose the bug - if manager is None, _run_with_result will fail
+            try:
+                result = await client.arun(mock_config, no_raise=True)
+                # If we get here, check what we got
+                print(f"Got result: {result}, type: {type(result)}")
+                assert result is not None, "arun() returned None due to broken process() context manager"
+                assert isinstance(result, RunResult), f"Expected RunResult, got {type(result)}"
+            except Exception as e:
+                # This might happen if _run_with_result(None) fails
+                print(f"Exception occurred: {e}")
+                # Even with no_raise=True, we shouldn't get unhandled exceptions
+                assert False, f"arun() with no_raise=True should not raise exception: {e}"
+
+    def test_run_no_raise_false_with_error_should_raise(self, mock_config):
+        """Test run with no_raise=False should RAISE exception when there's an error."""
+        client = PalabraAI(client_id="test", client_secret="test")
+
+        # Mock arun() to fail directly
+        with patch.object(client, 'arun') as mock_arun:
+            mock_arun.side_effect = asyncio.TimeoutError("arun failed")
+
+            # This should RAISE exception, not return RunResult
+            with pytest.raises(asyncio.TimeoutError) as exc_info:
+                client.run(mock_config, no_raise=False)
+
+            assert "arun failed" in str(exc_info.value)
+
+    def test_run_no_raise_true_with_error_should_return_result(self, mock_config):
+        """Test run with no_raise=True should return RunResult(ok=False) when there's an error."""
+        client = PalabraAI(client_id="test", client_secret="test")
+
+        # Mock arun() to fail directly
+        with patch.object(client, 'arun') as mock_arun:
+            mock_arun.side_effect = asyncio.TimeoutError("arun failed")
+
+            result = client.run(mock_config, no_raise=True)
+
+            assert result is not None
+            assert result.ok is False
+            assert result.exc is not None
+            assert isinstance(result.exc, asyncio.TimeoutError)
+            assert "arun failed" in str(result.exc)
+
 def test_run_signal_handlers_false():
     """Test run method with signal_handlers=False uses new event loop"""
     config = Config()
@@ -716,6 +911,79 @@ def test_run_and_arun_integration():
                 assert result == expected_result
 
 
+def test_client_uses_exception_for_manager_errors():
+    """Test that client.py uses exception() instead of error() for manager errors"""
+    config = Config()
+    config.source = SourceLang(lang="es")
+
+    client = PalabraAI(client_id="test", client_secret="test")
+
+    async def test_coro():
+        with patch.object(client, 'process') as mock_process:
+            mock_manager = MagicMock()
+            mock_manager.io = MagicMock()
+            mock_manager.io.io_data = {}
+            mock_manager.io.eos_received = False
+            mock_task = asyncio.Future()
+            mock_task.set_exception(ValueError("Manager error"))
+            mock_manager.task = mock_task
+            mock_manager.logger = MagicMock()
+            mock_manager.logger._task = MagicMock()
+            mock_manager.logger._task.done.return_value = True
+            mock_manager.logger.result = None
+            mock_process.return_value.__aenter__.return_value = mock_manager
+            mock_process.return_value.__aexit__.return_value = None
+
+            with patch('palabra_ai.client.exception') as mock_exception:
+                result = await client.arun(config, no_raise=True)
+                # Should use exception() for logging with traceback
+                mock_exception.assert_any_call("Error in manager task")
+                assert result.ok is False
+
+    asyncio.run(test_coro())
+
+
+def test_client_uses_exception_for_arun_errors():
+    """Test that client.py uses exception() for arun errors"""
+    config = Config()
+    config.source = SourceLang(lang="es")
+
+    client = PalabraAI(client_id="test", client_secret="test")
+
+    async def test_coro():
+        with patch.object(client, 'process') as mock_process:
+            mock_process.side_effect = RuntimeError("Process error")
+
+            with patch('palabra_ai.client.exception') as mock_exception:
+                result = await client.arun(config, no_raise=True)
+                # Should use exception() for logging with traceback
+                mock_exception.assert_called_once_with("Error in PalabraAI.arun()")
+                assert result.ok is False
+
+    asyncio.run(test_coro())
+
+
+def test_client_uses_exception_for_run_errors():
+    """Test that run() uses exception() for errors"""
+    config = Config()
+    config.source = SourceLang(lang="es")
+
+    client = PalabraAI(client_id="test", client_secret="test")
+
+    with patch('asyncio.new_event_loop') as mock_new_loop, \
+         patch('asyncio.set_event_loop') as mock_set_loop:
+
+        mock_loop = MagicMock()
+        mock_loop.run_until_complete.side_effect = ValueError("Test error")
+        mock_new_loop.return_value = mock_loop
+
+        with patch('palabra_ai.client.exception') as mock_exception:
+            result = client.run(config, no_raise=True)
+            # Should use exception() for logging with traceback
+            mock_exception.assert_called_once_with("An error occurred during execution")
+            assert result.ok is False
+
+
 @pytest.mark.asyncio
 async def test_run_result_eos_field_from_manager():
     """Test that RunResult gets eos field from manager.io.eos_received"""
@@ -729,6 +997,7 @@ async def test_run_result_eos_field_from_manager():
     mock_manager = MagicMock()
     mock_io = MagicMock()
     mock_io.eos_received = True
+    mock_io.io_data = {"start_perf_ts": 0.0, "start_utc_ts": 0.0, "in_sr": 16000, "out_sr": 16000, "mode": "test", "channels": 1, "events": [], "count_events": 0}
     mock_manager.io = mock_io
 
     # Mock logger with result
@@ -807,6 +1076,7 @@ async def test_run_result_eos_field_false():
     mock_manager = MagicMock()
     mock_io = MagicMock()
     mock_io.eos_received = False
+    mock_io.io_data = {"start_perf_ts": 0.0, "start_utc_ts": 0.0, "in_sr": 16000, "out_sr": 16000, "mode": "test", "channels": 1, "events": [], "count_events": 0}
     mock_manager.io = mock_io
 
     # Mock logger with result
@@ -831,4 +1101,190 @@ async def test_run_result_eos_field_false():
         # Result should have eos=False from manager.io.eos_received
         assert isinstance(result, RunResult)
         assert result.eos is False
-        assert result.ok is True
\ No newline at end of file
+        assert result.ok is True
+
+
+def test_benchmark_completes_successfully_with_graceful_shutdown():
+    """Test that benchmark completes successfully with graceful shutdown (integration-style test)"""
+    from palabra_ai.model import RunResult, IoData
+
+    config = Config()
+    config.source = SourceLang(lang="es")
+    config.targets = [TargetLang(lang="en")]
+
+    client = PalabraAI(client_id="test", client_secret="test")
+
+    # Create successful RunResult with io_data
+    io_data = IoData(
+        start_perf_ts=0.0,
+        start_utc_ts=0.0,
+        in_sr=16000,
+        out_sr=24000,
+        mode="ws",
+        channels=1,
+        events=[],
+        count_events=0
+    )
+
+    successful_result = RunResult(
+        ok=True,
+        exc=None,
+        io_data=io_data,
+        log_data=None,
+        eos=True
+    )
+
+    with patch.object(client, 'arun', return_value=successful_result) as mock_arun:
+        result = client.run(config, no_raise=True)
+
+        # Should complete successfully
+        assert result is not None
+        assert result.ok is True
+        assert result.io_data is not None
+
+
+@pytest.mark.asyncio
+async def test_manager_cancelled_graceful_shutdown():
+    """Test that graceful shutdown (manager._graceful_completion=True) does NOT log ERROR"""
+    config = Config()
+    config.source = SourceLang(lang="es")
+    config.targets = [TargetLang(lang="en")]
+
+    client = PalabraAI(client_id="test", client_secret="test")
+
+    with patch.object(client, 'process') as mock_process:
+        # Create manager with graceful completion flag
+        mock_manager = MagicMock()
+        mock_manager._graceful_completion = True  # GRACEFUL shutdown
+        mock_io = MagicMock()
+        mock_io.io_data = {"start_perf_ts": 0.0, "start_utc_ts": 0.0, "in_sr": 16000, "out_sr": 16000, "mode": "ws", "channels": 1, "events": [], "count_events": 0}
+        mock_io.eos_received = True
+        mock_manager.io = mock_io
+
+        # Manager task raises CancelledError (normal during graceful shutdown)
+        mock_task = asyncio.Future()
+        mock_task.set_exception(asyncio.CancelledError())
+        mock_manager.task = mock_task
+
+        mock_manager.logger = MagicMock()
+        mock_manager.logger._task = MagicMock()
+        mock_manager.logger._task.done.return_value = True
+        mock_manager.logger.result = None
+
+        mock_process.return_value.__aenter__.return_value = mock_manager
+        mock_process.return_value.__aexit__.return_value = None
+
+        with patch('palabra_ai.client.exception') as mock_exception, \
+             patch('palabra_ai.client.debug') as mock_debug:
+
+            result = await client.arun(config, no_raise=True)
+
+            # CRITICAL: Should NOT log as ERROR for graceful shutdown
+            # Check that exception() was NOT called with "Manager task was cancelled"
+            exception_calls = [call[0][0] for call in mock_exception.call_args_list]
+            assert "Manager task was cancelled" not in exception_calls, \
+                "Graceful shutdown should NOT log 'Manager task was cancelled' as ERROR"
+
+            # Should use debug() instead
+            debug_calls = [call[0][0] for call in mock_debug.call_args_list]
+            # We expect some debug message about graceful completion
+
+            # After fix: graceful shutdown should return ok=True, not False!
+            assert result.ok is True, "Graceful shutdown should have ok=True"
+            assert result.exc is None, "Graceful shutdown should have exc=None"
+
+
+@pytest.mark.asyncio
+async def test_manager_cancelled_external():
+    """Test that external cancellation (manager._graceful_completion=False) DOES log ERROR"""
+    config = Config()
+    config.source = SourceLang(lang="es")
+    config.targets = [TargetLang(lang="en")]
+
+    client = PalabraAI(client_id="test", client_secret="test")
+
+    with patch.object(client, 'process') as mock_process:
+        # Create manager WITHOUT graceful completion flag
+        mock_manager = MagicMock()
+        mock_manager._graceful_completion = False  # EXTERNAL cancellation
+        mock_io = MagicMock()
+        mock_io.io_data = {"start_perf_ts": 0.0, "start_utc_ts": 0.0, "in_sr": 16000, "out_sr": 16000, "mode": "ws", "channels": 1, "events": [], "count_events": 0}
+        mock_io.eos_received = False
+        mock_manager.io = mock_io
+
+        # Manager task raises CancelledError (external - Ctrl+C, timeout, error)
+        mock_task = asyncio.Future()
+        mock_task.set_exception(asyncio.CancelledError())
+        mock_manager.task = mock_task
+
+        mock_manager.logger = MagicMock()
+        mock_manager.logger._task = MagicMock()
+        mock_manager.logger._task.done.return_value = True
+        mock_manager.logger.result = None
+
+        mock_process.return_value.__aenter__.return_value = mock_manager
+        mock_process.return_value.__aexit__.return_value = None
+
+        with patch('palabra_ai.client.exception') as mock_exception:
+            result = await client.arun(config, no_raise=True)
+
+            # CRITICAL: SHOULD log as ERROR for external cancellation
+            mock_exception.assert_any_call("Manager task was cancelled")
+
+            # Result should indicate cancellation happened
+            assert result.ok is False
+            assert isinstance(result.exc, asyncio.CancelledError)
+
+
+@pytest.mark.asyncio
+async def test_graceful_shutdown_returns_ok_true():
+    """
+    Test that graceful shutdown (manager._graceful_completion=True) returns ok=True.
+    This is THE BUG we're fixing - graceful shutdown should NOT be treated as error!
+    """
+    config = Config()
+    config.source = SourceLang(lang="es")
+    config.targets = [TargetLang(lang="en")]
+
+    client = PalabraAI(client_id="test", client_secret="test")
+
+    with patch.object(client, 'process') as mock_process:
+        # Create manager with graceful completion flag
+        mock_manager = MagicMock()
+        mock_manager._graceful_completion = True  # GRACEFUL shutdown - normal EOF completion
+        mock_io = MagicMock()
+        mock_io.io_data = {
+            "start_perf_ts": 0.0,
+            "start_utc_ts": 0.0,
+            "in_sr": 16000,
+            "out_sr": 16000,
+            "mode": "ws",
+            "channels": 1,
+            "events": [],
+            "count_events": 0
+        }
+        mock_io.eos_received = True  # EOS was received - normal completion
+        mock_manager.io = mock_io
+
+        # Manager task raises CancelledError (due to shutdown_task timeout, but it's graceful)
+        mock_task = asyncio.Future()
+        mock_task.set_exception(asyncio.CancelledError())
+        mock_manager.task = mock_task
+
+        mock_manager.logger = MagicMock()
+        mock_manager.logger._task = MagicMock()
+        mock_manager.logger._task.done.return_value = True
+        mock_manager.logger.result = None
+
+        mock_process.return_value.__aenter__.return_value = mock_manager
+        mock_process.return_value.__aexit__.return_value = None
+
+        result = await client.arun(config, no_raise=True)
+
+        # CRITICAL: Graceful shutdown should return ok=True!
+        # This is normal completion - FileReader reached EOF, everything worked correctly
+        assert result is not None, "Result should not be None"
+        assert result.ok is True, "Graceful shutdown should have ok=True, not False!"
+        assert result.exc is None, "Graceful shutdown should have exc=None, not CancelledError!"
+        assert result.io_data is not None, "Should have io_data from successful run"
+        assert result.eos is True, "Should have eos=True since it was graceful completion"
\ No newline at end of file
