@@ -22,7 +22,7 @@ def mock_config():
     config.mode.num_channels = 1
     config.mode.samples_per_channel = 160
     config.mode.chunk_duration_ms = 10
-    config.mode.for_audio_frame = (16000, 1, 160)
+    config.mode.for_input_audio_frame = (16000, 1, 160)
     config.benchmark = False
     return config
 
@@ -115,13 +115,13 @@ class TestWsIo:
             assert str(frame) in str(mock_debug.call_args[0][0])
 
     def test_new_frame(self, mock_config, mock_credentials, mock_reader, mock_writer):
-        """Test new_frame method"""
+        """Test new_input_frame method"""
         ws_io = WsIo(cfg=mock_config, credentials=mock_credentials, reader=mock_reader, writer=mock_writer)
 
         with patch('palabra_ai.audio.AudioFrame.create') as mock_create:
             mock_create.return_value = MagicMock(spec=AudioFrame)
 
-            frame = ws_io.new_frame()
+            frame = ws_io.new_input_frame()
 
             mock_create.assert_called_once_with(16000, 1, 160)
             assert frame == mock_create.return_value
@@ -177,16 +177,17 @@ class TestWsIo:
 
         with patch('palabra_ai.audio.AudioFrame.from_ws', return_value=None):
             with patch('palabra_ai.message.Message.decode', return_value=mock_msg):
-                with patch('palabra_ai.task.io.ws.debug'):
-                    with patch('palabra_ai.task.io.ws.trace'):
-                        try:
-                            await ws_io.ws_receiver()
-                        except EOFError:
-                            pass  # Expected when no more messages
+                with patch('palabra_ai.task.io.ws.IoEvent'):
+                    with patch('palabra_ai.task.io.ws.debug'):
+                        with patch('palabra_ai.task.io.ws.trace'):
+                            try:
+                                await ws_io.ws_receiver()
+                            except EOFError:
+                                pass  # Expected when no more messages
 
-                        # Check message was published
-                        ws_io.out_msg_foq.publish.assert_any_call(mock_msg)
-                        assert mock_msg._dbg is not None
+                            # Check message was published
+                            ws_io.out_msg_foq.publish.assert_any_call(mock_msg)
+                            assert mock_msg._dbg is not None
 
     @pytest.mark.asyncio
     async def test_ws_receiver_eos_message(self, mock_config, mock_credentials, mock_reader, mock_writer):
@@ -209,13 +210,14 @@ class TestWsIo:
 
         with patch('palabra_ai.audio.AudioFrame.from_ws', return_value=None):
             with patch('palabra_ai.message.Message.decode', return_value=mock_eos):
-                with patch('palabra_ai.task.io.ws.debug') as mock_debug:
-                    with patch('palabra_ai.task.io.ws.trace'):
-                        await ws_io.ws_receiver()
+                with patch('palabra_ai.task.io.ws.IoEvent'):
+                    with patch('palabra_ai.task.io.ws.debug') as mock_debug:
+                        with patch('palabra_ai.task.io.ws.trace'):
+                            await ws_io.ws_receiver()
 
-                        # Check EOF was set
-                        assert ws_io.eof.is_set()
-                        mock_debug.assert_any_call(f"EOF!!! End of stream received: {mock_eos}")
+                            # Check EOF was set
+                            assert ws_io.eof.is_set()
+                            mock_debug.assert_any_call(f"EOF!!! End of stream received: {mock_eos}")
 
     @pytest.mark.asyncio
     async def test_ws_receiver_benchmark_mode(self, mock_config, mock_credentials, mock_reader, mock_writer):
@@ -239,17 +241,18 @@ class TestWsIo:
         mock_frame = MagicMock(spec=AudioFrame)
 
         with patch('palabra_ai.audio.AudioFrame.from_ws', return_value=mock_frame):
-            with patch('palabra_ai.task.io.ws.debug'):
-                with patch('palabra_ai.task.io.ws.trace'):
-                    with patch('asyncio.to_thread', new_callable=AsyncMock, return_value=-20.0):
-                        try:
-                            await ws_io.ws_receiver()
-                        except EOFError:
-                            pass
+            with patch('palabra_ai.task.io.ws.IoEvent'):
+                with patch('palabra_ai.task.io.ws.debug'):
+                    with patch('palabra_ai.task.io.ws.trace'):
+                        with patch('asyncio.to_thread', new_callable=AsyncMock, return_value=-20.0):
+                            try:
+                                await ws_io.ws_receiver()
+                            except EOFError:
+                                pass
 
-                        # Check benchmark data was added
-                        assert mock_frame._dbg is not None
-                        ws_io.bench_audio_foq.publish.assert_called_once_with(mock_frame)
+                            # Check benchmark data was added
+                            assert mock_frame._dbg is not None
+                            ws_io.bench_audio_foq.publish.assert_called_once_with(mock_frame)
 
     @pytest.mark.asyncio
     async def test_boot(self, mock_config, mock_credentials, mock_reader, mock_writer):
@@ -343,14 +346,15 @@ class TestWsIo:
 
         with patch('palabra_ai.audio.AudioFrame.from_ws', return_value=None):
             with patch('palabra_ai.message.Message.decode', return_value=mock_eos):
-                with patch('palabra_ai.task.io.ws.debug'):
-                    with patch('palabra_ai.task.io.ws.trace'):
-                        await ws_io.ws_receiver()
+                with patch('palabra_ai.task.io.ws.IoEvent'):
+                    with patch('palabra_ai.task.io.ws.debug'):
+                        with patch('palabra_ai.task.io.ws.trace'):
+                            await ws_io.ws_receiver()
 
-                        # Check eos_received was set to True
-                        assert ws_io.eos_received is True
-                        # Check EOF was also set
-                        assert ws_io.eof.is_set()
+                            # Check eos_received was set to True
+                            assert ws_io.eos_received is True
+                            # Check EOF was also set
+                            assert ws_io.eof.is_set()
 
     @pytest.mark.asyncio
     async def test_ws_receiver_eof_exception_sets_flag(self, mock_config, mock_credentials, mock_reader, mock_writer):
@@ -375,4 +379,82 @@ class TestWsIo:
             # Check eos_received was set to True in exception handler
             assert ws_io.eos_received is True
             # Check EOF was also set
-            assert ws_io.eof.is_set()
\ No newline at end of file
+            assert ws_io.eof.is_set()
+
+    @pytest.mark.asyncio
+    async def test_send_frame_uses_raw_when_provided(self, mock_config, mock_credentials, mock_reader, mock_writer):
+        """Test send_frame uses provided raw without calling frame.to_ws()"""
+        ws_io = WsIo(cfg=mock_config, credentials=mock_credentials, reader=mock_reader, writer=mock_writer)
+        ws_io.ws = AsyncMock()
+
+        # Mock frame
+        frame = MagicMock(spec=AudioFrame)
+        frame.to_ws = MagicMock(return_value=b"should_not_be_called")
+
+        # Provide raw bytes
+        raw_bytes = b"provided_raw_data"
+
+        with patch('palabra_ai.task.io.ws.debug'):
+            await ws_io.send_frame(frame, raw=raw_bytes)
+
+        # frame.to_ws() should NOT be called (optimization)
+        frame.to_ws.assert_not_called()
+
+        # ws.send() should be called with provided raw
+        ws_io.ws.send.assert_called_once_with(raw_bytes)
+
+    @pytest.mark.asyncio
+    async def test_send_frame_converts_when_raw_not_provided(self, mock_config, mock_credentials, mock_reader, mock_writer):
+        """Test send_frame calls frame.to_ws() when raw not provided"""
+        ws_io = WsIo(cfg=mock_config, credentials=mock_credentials, reader=mock_reader, writer=mock_writer)
+        ws_io.ws = AsyncMock()
+
+        # Mock frame
+        frame = MagicMock(spec=AudioFrame)
+        converted_bytes = b"converted_data"
+        frame.to_ws = MagicMock(return_value=converted_bytes)
+
+        with patch('palabra_ai.task.io.ws.debug'):
+            await ws_io.send_frame(frame, raw=None)
+
+        # frame.to_ws() should be called
+        frame.to_ws.assert_called_once()
+
+        # ws.send() should be called with converted data
+        ws_io.ws.send.assert_called_once_with(converted_bytes)
+
+    @pytest.mark.asyncio
+    async def test_push_benchmark_mode_passes_raw(self, mock_config, mock_credentials, mock_reader, mock_writer):
+        """Test push() in benchmark mode passes raw to send_frame()"""
+        import numpy as np
+
+        mock_config.mode.input_samples_per_channel = 160
+        mock_config.mode.for_input_audio_frame = (16000, 1, 160)
+        mock_config.benchmark = True
+
+        ws_io = WsIo(cfg=mock_config, credentials=mock_credentials, reader=mock_reader, writer=mock_writer)
+        ws_io._idx = iter(range(100))
+        ws_io._in_audio_num = iter(range(100))
+        ws_io.bench_audio_foq = MagicMock()
+
+        # Mock send_frame to capture arguments
+        ws_io.send_frame = AsyncMock()
+
+        # Create audio bytes
+        audio_bytes = np.random.randint(-32768, 32767, 160, dtype=np.int16).tobytes()
+
+        with patch('palabra_ai.task.io.ws.IoEvent'):
+            with patch('asyncio.to_thread', new_callable=AsyncMock, return_value=-20.0):
+                await ws_io.push(audio_bytes)
+
+        # send_frame should be called once
+        assert ws_io.send_frame.call_count == 1
+
+        # Check that raw parameter is NOT None (benchmark mode optimization)
+        call_args = ws_io.send_frame.call_args
+        frame_arg = call_args[0][0]
+        raw_arg = call_args[0][1] if len(call_args[0]) > 1 else call_args[1].get('raw')
+
+        # raw should be bytes (not None) in benchmark mode
+        assert raw_arg is not None
+        assert isinstance(raw_arg, bytes)
\ No newline at end of file
